package java.awt;

import javax.accessibility.AccessibleRole;
import sun.awt.AWTAccessor;
import javax.accessibility.AccessibleContext;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.awt.event.KeyEvent;
import java.util.Enumeration;
import java.awt.peer.MenuBarPeer;
import java.util.Vector;
import javax.accessibility.Accessible;

public class MenuBar extends MenuComponent implements MenuContainer, Accessible
{
    Vector<Menu> menus;
    Menu helpMenu;
    private static final String base = "menubar";
    private static int nameCounter;
    private static final long serialVersionUID = -4930327919388951260L;
    private int menuBarSerializedDataVersion;
    
    public MenuBar() throws HeadlessException {
        this.menus = new Vector();
        this.menuBarSerializedDataVersion = 1;
    }
    
    @Override
    String constructComponentName() {
        synchronized (MenuBar.class) {
            return "menubar" + MenuBar.nameCounter++;
        }
    }
    
    public void addNotify() {
        synchronized (this.getTreeLock()) {
            if (this.peer == null) {
                this.peer = Toolkit.getDefaultToolkit().createMenuBar(this);
            }
            for (int menuCount = this.getMenuCount(), i = 0; i < menuCount; ++i) {
                this.getMenu(i).addNotify();
            }
        }
    }
    
    @Override
    public void removeNotify() {
        synchronized (this.getTreeLock()) {
            for (int menuCount = this.getMenuCount(), i = 0; i < menuCount; ++i) {
                this.getMenu(i).removeNotify();
            }
            super.removeNotify();
        }
    }
    
    public Menu getHelpMenu() {
        return this.helpMenu;
    }
    
    public void setHelpMenu(Menu helpMenu) {
        synchronized (this.getTreeLock()) {
            if (this.helpMenu == helpMenu) {
                return;
            }
            if (this.helpMenu != null) {
                this.remove(this.helpMenu);
            }
            if ((this.helpMenu = helpMenu) != null) {
                if (helpMenu.parent != this) {
                    this.add(helpMenu);
                }
                helpMenu.isHelpMenu = true;
                helpMenu.parent = this;
                MenuBarPeer menuBarPeer = (MenuBarPeer)this.peer;
                if (menuBarPeer != null) {
                    if (helpMenu.peer == null) {
                        helpMenu.addNotify();
                    }
                    menuBarPeer.addHelpMenu(helpMenu);
                }
            }
        }
    }
    
    public Menu add(Menu menu) {
        synchronized (this.getTreeLock()) {
            if (menu.parent != null) {
                menu.parent.remove(menu);
            }
            menu.parent = this;
            MenuBarPeer menuBarPeer = (MenuBarPeer)this.peer;
            if (menuBarPeer != null) {
                if (menu.peer == null) {
                    menu.addNotify();
                }
                this.menus.addElement(menu);
                menuBarPeer.addMenu(menu);
            }
            else {
                this.menus.addElement(menu);
            }
            return menu;
        }
    }
    
    public void remove(int index) {
        synchronized (this.getTreeLock()) {
            Menu menu = this.getMenu(index);
            this.menus.removeElementAt(index);
            MenuBarPeer menuBarPeer = (MenuBarPeer)this.peer;
            if (menuBarPeer != null) {
                menuBarPeer.delMenu(index);
                menu.removeNotify();
                menu.parent = null;
            }
            if (this.helpMenu == menu) {
                this.helpMenu = null;
                menu.isHelpMenu = false;
            }
        }
    }
    
    @Override
    public void remove(MenuComponent o) {
        synchronized (this.getTreeLock()) {
            int index = this.menus.indexOf(o);
            if (index >= 0) {
                this.remove(index);
            }
        }
    }
    
    public int getMenuCount() {
        return this.countMenus();
    }
    
    @Deprecated
    public int countMenus() {
        return this.getMenuCountImpl();
    }
    
    final int getMenuCountImpl() {
        return this.menus.size();
    }
    
    public Menu getMenu(int n) {
        return this.getMenuImpl(n);
    }
    
    final Menu getMenuImpl(int index) {
        return (Menu)this.menus.elementAt(index);
    }
    
    public synchronized Enumeration<MenuShortcut> shortcuts() {
        Vector<Object> vector = new Vector();
        for (int menuCount = this.getMenuCount(), i = 0; i < menuCount; ++i) {
            Enumeration<MenuShortcut> shortcuts = this.getMenu(i).shortcuts();
            while (shortcuts.hasMoreElements()) {
                vector.addElement(shortcuts.nextElement());
            }
        }
        return vector.elements();
    }
    
    public MenuItem getShortcutMenuItem(MenuShortcut menuShortcut) {
        int menuCount = this.getMenuCount();
        int i = 0;
        while (i < menuCount) {
            MenuItem shortcutMenuItem = this.getMenu(i).getShortcutMenuItem(menuShortcut);
            if (shortcutMenuItem != null) {
                return shortcutMenuItem;
            }
            ++i;
        }
        return null;
    }
    
    boolean handleShortcut(KeyEvent keyEvent) {
        int id = keyEvent.getID();
        if (id != 401 && id != 402) {
            return false;
        }
        if ((keyEvent.getModifiers() & Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()) == 0x0) {
            return false;
        }
        int menuCount = this.getMenuCount();
        int i = 0;
        while (i < menuCount) {
            if (this.getMenu(i).handleShortcut(keyEvent)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public void deleteShortcut(MenuShortcut menuShortcut) {
        for (int menuCount = this.getMenuCount(), i = 0; i < menuCount; ++i) {
            this.getMenu(i).deleteShortcut(menuShortcut);
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws ClassNotFoundException, IOException {
        objectOutputStream.defaultWriteObject();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException, HeadlessException {
        objectInputStream.defaultReadObject();
        for (int i = 0; i < this.menus.size(); ++i) {
            ((Menu)this.menus.elementAt(i)).parent = this;
        }
    }
    
    private static native void initIDs();
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new MenuBar.AccessibleAWTMenuBar();
        }
        return this.accessibleContext;
    }
    
    @Override
    int getAccessibleChildIndex(MenuComponent o) {
        return this.menus.indexOf(o);
    }
    
    static {
        Toolkit.loadLibraries();
        if (!GraphicsEnvironment.isHeadless()) {
            initIDs();
        }
        AWTAccessor.setMenuBarAccessor(new AWTAccessor.MenuBarAccessor() {
            @Override
            public Menu getHelpMenu(MenuBar menuBar) {
                return menuBar.helpMenu;
            }
            
            @Override
            public Vector<Menu> getMenus(MenuBar menuBar) {
                return menuBar.menus;
            }
        });
        MenuBar.nameCounter = 0;
    }
    
    protected class AccessibleAWTMenuBar extends MenuComponent.AccessibleAWTMenuComponent
    {
        private static final long serialVersionUID = -8577604491830083815L;
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.MENU_BAR;
        }
    }
}
